/* aguixfont.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2006 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "aguixfont.hh"
#include "aguix.h"

AGUIXFont::AGUIXFont(AGUIX *parent)
{
  name=NULL;
  gc=0;
  font=NULL;
  aguix=parent;
}

AGUIXFont::~AGUIXFont()
{
  removeFont();
}

void AGUIXFont::removeFont()
{
  if(gc!=0) {
    XFreeGC(aguix->getDisplay(),gc);
    gc=0;
  }
  if(font!=NULL) {
    XFreeFont(aguix->getDisplay(),font);
    font=NULL;
  }
  if(name!=NULL) {
    _freesafe(name);
    name=NULL;
  }
}

int AGUIXFont::setFont( const char *fontname )
{
  Display *dsp=aguix->getDisplay();
  int scr=aguix->getScreen();
  unsigned long white=WhitePixel(dsp,scr);
  unsigned long black=BlackPixel(dsp,scr);
  gc=XCreateGC(dsp,RootWindow(dsp,scr),0,0);
  XGCValues gc_values;
  gc_values.graphics_exposures=False;
  gc_values.foreground=black;
  gc_values.background=white;
  gc_values.line_width=0;
  gc_values.cap_style=CapButt;
  font=XLoadQueryFont(dsp,fontname);
  if(font==NULL) {
    XFreeGC(dsp,gc);
    gc=0;
    return 1;
  }
  charHeight=font->ascent+font->descent;
  gc_values.font=font->fid;
  unsigned long gc_valuemask=GCGraphicsExposures|GCForeground|GCBackground|GCLineWidth|GCFont|GCCapStyle;
  XChangeGC(dsp,gc,gc_valuemask,&gc_values);
  if(name!=NULL) _freesafe(name);
  name=dupstring(fontname);
  return 0;
}

const char *AGUIXFont::getName() const
{
  return name;
}

GC AGUIXFont::getGC() const
{
  return gc;
}

int AGUIXFont::getMaxCharWidth() const
{
#if 1
  return font->max_bounds.rbearing - font->min_bounds.lbearing;
#else
  return textWidth( "M", 1 );
#endif
}

int AGUIXFont::getCharHeight() const
{
  return charHeight;
}

int AGUIXFont::getBaseline() const
{
  return font->ascent;
}

int AGUIXFont::textWidth( const char *str, int count ) const
{
  return XTextWidth( font, str, count );
}

Font AGUIXFont::getFont() const
{
  return font->fid;
}

void AGUIXFont::drawText( Drawable buffer, const char *text, int x, int y, int color )
{
  aguix->setFG( gc, color );
  XDrawString( aguix->getDisplay(),
	       buffer,
	       gc,
	       x,
	       y + getBaseline(),
	       text,
	       strlen( text ) );
}
